"""
UpLang - Minecraft Modpack Language File Synchronizer

A command-line tool to streamline the process of updating language files
for Minecraft Java Edition modpacks.
"""

__version__ = "0.3.0"
__author__ = "QianFuv"
__email__ = "qianfuv@qq.com"
__license__ = "MIT"

__all__ = ["__version__", "__author__", "__email__", "__license__"]
